use strict;
use warnings;
use Test::More tests => 5;

require_ok 'Test::Alien';
require_ok 'Test::Alien::CanCompile';
require_ok 'Test::Alien::CanPlatypus';
require_ok 'Test::Alien::Run';
require_ok 'Test::Alien::Synthetic';
